/* handling functions for cdrom drives installed in Gnome Toaster */

#include <gtk/gtk.h>
#include <string.h>

#include "cddrives.h"
#include "datatrack.h"
#include "audiotrack.h"
#include "streams.h"
#include "preferences.h"
#include "varman.h"
#include "varmanwidgets.h"
#include "tracks.h"
#include "record.h"

cddrives_drives_def cddrives_drives;

void cddrives_updatedrivelist();
int cddrives_checkcdchange(gpointer);

void cddrives_enablecdchangedetection()
{
 	gtk_timeout_add(1000,
			cddrives_checkcdchange,
			NULL);
}
;
	
/* install a preferences page in setup to enter a list of valid cd drives */
void cddrives_init()
{
	GtkWidget *cddrives_prefs;
	GtkWidget *label;
	
	varmanwidgets_widget *dlist;
	
	cddrives_prefs=gtk_vbox_new(0,0);
	gtk_widget_show(cddrives_prefs);
	label=gtk_label_new("CDROM Drive Setup");
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(preferences_editpad),
				 cddrives_prefs,
				 label);
	
	dlist=varmanwidgets_entry_new("valid drives:",
				      "cddrives_drives",
				      global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(cddrives_prefs),dlist->visual,0,0,0);
	
	cddrives_createdrivelist();
	
	varman_install_handler(global_defs,"cddrives_drives",
			       cddrives_updatedrivelist);

	cddrives_enablecdchangedetection();
}
;

void cddrives_cdinfo_update(cddrives_cdinfo *info)
{
	int tracksoncd;
	int trackcount;
	int isrecorder;
	
	isrecorder=(strcmp((char*)&info->device,RECORDER)==0);

	info->cddb=cdromlow_cddbnumber((char*)&info->device);
	info->tracks=0;
	tracksoncd=cdromlow_tracks((char*)&info->device);
	if (tracksoncd>0)
	  {		  
		  if (cdromlow_hasdatatrack((char*)&info->device))
		    {
			    info->track[0]=datatrack_create((char*)&info->device);
			    if (isrecorder)
				info->track[0]->precacherequired=atadd;
			    info->tracks++;
		    }
		  ;
		  for (trackcount=info->tracks;trackcount<tracksoncd;trackcount++)
		    {
			    info->track[trackcount]=audiotrack_create((char*)&info->device,trackcount+1);
			    /* if the track is located on the current cd writer
			     * force precaching */
			    if (isrecorder)
				info->track[trackcount]->precacherequired=atadd;
			    info->tracks++;
		    }
		  ;
	  }
	;
}
;

/* wrapper creating a new device */
cddrives_cdinfo *cddrives_cdinfo_create(char *device)
{
	cddrives_cdinfo *info;
	
	info=(cddrives_cdinfo*)malloc(sizeof(cddrives_cdinfo));
	strcpy((char*)&info->device,device);
	cddrives_cdinfo_update(info);
	
	return info;
}
;

void cddrives_cdinfo_destroycontent(cddrives_cdinfo *info)
{
	int x;
	
	for (x=0;x<info->tracks;x++)
	  {
		  tracks_unclaim(info->track[x]);
	  }
	;
}
;

void cddrives_cdinfo_destroy(cddrives_cdinfo *info)
{
	cddrives_cdinfo_destroycontent(info);
        free(info);
}
;

void  cddrives_createdrivelist()
{
	cddrives_cdinfo *info;
	char *drives,*cdrive,*ndrive;

	/* get drive list */
	drives=varman_getvar_copy(global_defs,"cddrives_drives");
	/* this is done to make the while loop easier */
	if (drives[strlen(drives)]!=" "[0]) strcat(drives," ");
	
	cdrive=drives;
	cddrives_drives.drivecount=0;
	
	while (strchr(cdrive," "[0])!=NULL)
	  {		  
		  ndrive=strchr(cdrive," "[0]);
		  *ndrive=0;		  
		  
		  info=cddrives_cdinfo_create(cdrive);
		  cddrives_drives.drive[cddrives_drives.drivecount]=info;
		  
		  cddrives_drives.drivecount++;
		  cdrive=ndrive+1;		  
	  };
	free(drives);
}
;
	
void  cddrives_destroydrivelist()
{
	int x;

	for (x=0;x<cddrives_drives.drivecount;x++)
	  {
		  cddrives_cdinfo_destroy(cddrives_drives.drive[x]);
	  }
	;
	cddrives_drives.drivecount=0;
}
;

void cddrives_updatedrivelist()
{
	cddrives_destroydrivelist();
	cddrives_createdrivelist();
}
;

int cddrives_checkcdchange(gpointer d)
{
	int x;
	char name[256];

	if (!record_active)
	  {		  
		  for (x=0;x<cddrives_drives.drivecount;x++)
		    {
			    if (cddrives_drives.drive[x]->cddb!=
				cdromlow_cddbnumber((char*)&cddrives_drives.drive[x]->device))
			      {
				      cddrives_cdinfo_destroycontent(cddrives_drives.drive[x]);
				      cddrives_cdinfo_update(cddrives_drives.drive[x]);
			      }
			    ;
		    }
		  ;
	  }
	;
	return 1; /* keep the timeout running */
}
;

/* return rom containing cd with cddb code */
cddrives_cdinfo *cddrives_getrombycddb(int cddb)
{
	int x;
	cddrives_cdinfo *f=NULL;	
	for (x=0;(x<cddrives_drives.drivecount)&&(f==NULL);x++)
	  {
		  if (cddrives_drives.drive[x]->cddb==cddb)
		      f=cddrives_drives.drive[x];
	  }
	;
	return f;
}
;

