/* this file collects some functions managing those parts of a progress
 * dialog appearing whenever files are being copied,that are the same on
 * each such occasion like a throuput display,a display for the time
 * remaining,one for the time already passed */

#include <gtk/gtk.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "datacopydlg.h"
	
void datacopydlg_destroy(datacopydlg_dlginfo *dlg)
{
	gtk_grab_remove(dlg->messagebox);
	gtk_widget_destroy(dlg->messagebox);
	free(dlg);
}
;

/* update the informations in datacopydlg_dlginfo and display changes 
 * this function should be called whenever a new block of data has been
 * processed */
void datacopydlg_newdatablock(datacopydlg_dlginfo *dlg,int size)
{
	int newtime;
	char buf[256];
	int remaining;
	gfloat done;
	
	dlg->bytessincelastmeasure+=size;
	time((time_t*)&newtime);
	if (difftime(newtime,dlg->lastmeasure)>2)
	  {
		  dlg->currentthroughput=
		      ((float)dlg->bytessincelastmeasure/
		       difftime(newtime,dlg->lastmeasure)
		       );
		  dlg->bytessincelastmeasure=0;
		  dlg->lastmeasure=newtime;
		  strcpy((char*)&buf,"Throughput:");
		  sprintf((char*)&buf[strlen((char*)&buf)],
			  "%.2f",
			  dlg->currentthroughput/1024);
		  strcat((char*)&buf," kb/sec.\n");
		  strcat((char*)&buf," Time elapsed: ");
		  sprintf((char*)&buf[strlen((char*)&buf)],
			  "%.0f",
			  difftime(newtime,dlg->starttime));
		  strcat((char*)&buf," sec.\n");
		  remaining=((dlg->totalsize-dlg->bytesdone)/
			     dlg->currentthroughput);
		  if (remaining<0) remaining=0;
		  strcat((char*)&buf," Time remaining: ");
		  sprintf((char*)&buf[strlen((char*)&buf)],
			  "%d",
			  remaining);
		  strcat((char*)&buf," sec.\n");
			    
		  gtk_label_set_text(GTK_LABEL(dlg->infoline),
				     (char*)&buf);
			    
	  }
	;		  
	dlg->bytesdone+=size;
	done=(float)dlg->bytesdone/(float)dlg->totalsize;
	if (done>1) done=1;
	if (done<0) done=0;
	gtk_progress_bar_update(GTK_PROGRESS_BAR(dlg->progress),
				done);
}
;
       
/* create a progress box with title for processing totalsize bytes of data,
 * stophandler gets called whenever the user terminates the process by
 * clicking the stop button. if NULL,stop button will not even be visible
 * data can be a pointer to some additional data passed to the signal handler
 * but can also be set to NULL if not needed */
datacopydlg_dlginfo *datacopydlg_create(char *title,
					int totalsize,
					GtkSignalFunc stophandler,
					gpointer data)
{
	GtkWidget *stop;
	datacopydlg_dlginfo *dlg;
	
	dlg=(datacopydlg_dlginfo*)malloc(sizeof(datacopydlg_dlginfo));
	
	dlg->messagebox=gtk_dialog_new();
	gtk_grab_add(dlg->messagebox);
	if (stophandler!=NULL)
	  {		  
		  stop=gtk_button_new_with_label("Stop");
		  gtk_signal_connect(GTK_OBJECT(stop),"clicked",
				     stophandler,data);
		  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->messagebox)->action_area),
				     stop,TRUE,TRUE,0);
		  gtk_widget_show(stop);
	  }
	;	
	gtk_window_set_title(GTK_WINDOW(dlg->messagebox),
			     title);
	gtk_window_set_position(GTK_WINDOW(dlg->messagebox),
				GTK_WIN_POS_CENTER);
	dlg->infoline=gtk_label_new("please wait...");
	gtk_label_set_justify(GTK_LABEL(dlg->infoline),
			      GTK_JUSTIFY_LEFT);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->messagebox)->vbox),
			   dlg->infoline,TRUE,TRUE,0);
	gtk_widget_show(dlg->infoline);
	dlg->progress=gtk_progress_bar_new();
	gtk_progress_set_format_string(GTK_PROGRESS(dlg->progress),
				   "%p%% done");
	gtk_progress_set_show_text(GTK_PROGRESS(dlg->progress),
				   TRUE);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg->messagebox)->vbox),
			   dlg->progress,TRUE,TRUE,0);
	gtk_widget_show(dlg->progress);
	dlg->totalsize=totalsize;
        dlg->bytesdone=0;
	
	gtk_widget_show(dlg->messagebox);	

	time((time_t*)&dlg->lastmeasure);
	dlg->starttime=dlg->lastmeasure;	
	dlg->bytessincelastmeasure=0;
	
	return dlg;
}
;

	
