#include <unistd.h>
#include <getopt.h>
#include <stdlib.h>

#include <gtk/gtk.h>

#include "varman.h"
#include "varmanwidgets.h"
#include "preferences.h"

GtkWidget *preferences_editmain;
GtkWidget *preferences_editpad;
GtkWidget *preferences_edit_ok,*preferences_edit_apply,
          *preferences_edit_cancel,*preferences_edit_undo;

GtkWidget *preferences_common;
	
varman_database *global_defs;
/* all those things that shouldnt be saved to the config file */
varman_database *dynamic_defs;

struct option preflongoptions[]=
{
	  {"tempdir",required_argument,NULL,'t'},
	  {"cdfsdir",required_argument,NULL,'c'},
	  {"recorder",required_argument,NULL,'r'},
	  {"help",no_argument,NULL,'h'},
          {0,0,0,0}
};
		  

void display_help()
{
	printf("Available options:\n"
	       "\n"
	       " --tempdir  -t    Directory to save images and precached data\n"
	       " --cdfsdir  -c    Directory to build CD Filesystem in (only symlinks\n"
	       "                  will be created here)\n"
	       " --recorder -r    CD Recorder device\n"
	       " --help     -h    Display this text\n");
};

void preferences_edit_hide(GtkWidget *w,gpointer data)
{
	gtk_widget_hide(preferences_editmain);
}
;

void preferences_setapplyhandler(GtkSignalFunc handler,
				 gpointer data)
{
	gtk_signal_connect(GTK_OBJECT(preferences_edit_ok),"clicked",
			   handler,data);
	gtk_signal_connect(GTK_OBJECT(preferences_edit_apply),"clicked",
			   handler,data);
}
;

void preferences_setundohandler(GtkSignalFunc handler,
				gpointer data)
{
	gtk_signal_connect(GTK_OBJECT(preferences_edit_cancel),"clicked",
			   handler,data);
	gtk_signal_connect(GTK_OBJECT(preferences_edit_undo),"clicked",
			   handler,data);
}
;
	
void preferences_init(int argc,char **argv)
{
	int c;
	int arg;
	
	GtkWidget *label;
	
	varmanwidgets_widget *td;
	
	global_defs=varman_vardatabase_create_new(MAX_GLOBAL_DEFS);
	dynamic_defs=varman_vardatabase_create_new(MAX_DYNAMIC_DEFS);

	/* the most important environment variables */
	varman_setvar(dynamic_defs,"$HOME",getenv("HOME"));	

	/* default definitions */	
	varman_setvar(global_defs,"fsedit_destpath","/home/dos4gw/Burn");
	varman_setvar(global_defs,"tempdir","/dos/d/Burn");
	varman_setvar(global_defs,"recorder","/dev/scd0");
	varman_setvar(global_defs,"rec_scsiid","0,0");
	varman_setvar(global_defs,"rec_client","/usr/bin/cdrecord");
	varman_setvar(global_defs,"rec_fixate","rec_client dev=rec_scsiid speed=2 -fix");
	varman_setvar(global_defs,"rec_writetrack","rec_client dev=rec_scsiid speed=2 -nofix -$tracktype -");
	varman_setvar(global_defs,"rec_cleardisc","rec_client dev=rec_scsiid speed=2 -blank=rec_blankingmode");
	varman_setvar(global_defs,"rec_blankingmode","fast");
	varman_setvar(global_defs,"isotrack_client","mkhybrid -f -J -R fsedit_destpath");
	varman_setvar(global_defs,"isotrack_tsclient","du -b -s -l -L fsedit_destpath");
	varman_setvar(global_defs,"audiotrack_client","cdda2wav");
	varman_setvar(global_defs,"audiotrack_read","audiotrack_client -O cdr -D $device -t $tracknum -");
	varman_setvar(global_defs,"cddrives_drives","/dev/cdrom");

	/* load the GnomeToaster Configuration file,overwriting the defaults
	 * defined above if present */
	varman_loaddatabase(global_defs,
			    varman_replacevars_copy(dynamic_defs,"$HOME/.gtoasterrc"),
			    "[global_defs]");
	
	/* do process the command line *after* the config file sos 
	 * eventual parameters can enter the program and dont get overwritten
	 * by the configfile defaults */	
	while ((c=getopt_long(argc,argv,"t:c:r:h",preflongoptions,&arg))!=-1)
		{
			switch (c)
			  {
			  case 't': //--tempdir or -t
				  varman_setvar(global_defs,
						"tempdir",
						optarg);
				  break;
			  case 'c': //--cdfsdir or -c
				  varman_setvar(global_defs,
						"fsedit_destpath",
						optarg);
				  break;
			  case 'r': // --recorder or -r
				  varman_setvar(global_defs,
						"recorder",
						optarg);
				  break;
			  case 'h': // --help or -h
				  display_help();
				  exit(0);
			  };
		};
	
	
	preferences_editmain=gtk_dialog_new();
	gtk_widget_set_usize(preferences_editmain,500,300);
	gtk_window_set_title(GTK_WINDOW(preferences_editmain),"GnomeToaster Preferences");

	preferences_editpad=gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(preferences_editmain)->vbox),
			   preferences_editpad,TRUE,TRUE,1);
	gtk_widget_show(preferences_editpad);
	
	preferences_edit_apply=gtk_button_new_with_label("Apply");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(preferences_editmain)->action_area),
			   preferences_edit_apply,FALSE,FALSE,0);
	gtk_widget_show(preferences_edit_apply);
	
	preferences_edit_undo=gtk_button_new_with_label("Undo");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(preferences_editmain)->action_area),
			   preferences_edit_undo,FALSE,FALSE,0);
	gtk_widget_show(preferences_edit_undo);
	
	preferences_edit_ok=gtk_button_new_with_label("OK");
	gtk_signal_connect(GTK_OBJECT(preferences_edit_ok),"clicked",
			   preferences_edit_hide,NULL);	
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(preferences_editmain)->action_area),
			   preferences_edit_ok,FALSE,FALSE,0);
	gtk_widget_show(preferences_edit_ok);

	preferences_edit_cancel=gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(preferences_edit_cancel),"clicked",
			   preferences_edit_hide,NULL);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(preferences_editmain)->action_area),
			   preferences_edit_cancel,FALSE,FALSE,0);
	gtk_widget_show(preferences_edit_cancel);
	

	preferences_common=gtk_vbox_new(0,0);
	gtk_widget_show(preferences_common);
	label=gtk_label_new("Common");
	gtk_widget_show(label);
	gtk_notebook_append_page(GTK_NOTEBOOK(preferences_editpad),
				 preferences_common,
				 label);

	td=varmanwidgets_entry_new("Temp Files",
				   "tempdir",
				   global_defs,APPLYMODE_BUTTON,160);
	gtk_box_pack_start(GTK_BOX(preferences_common),td->visual,0,0,0);
	
};

/* store the config informations */
void preferences_exit()
{
	varman_savedatabase(global_defs,
			    varman_replacevars_copy(dynamic_defs,"$HOME/.gtoasterrc"),
			    "[global_defs]");
}
;
