/* The track editor */

#include <gtk/gtk.h>

#include "main.h"
#include "tracks.h"
#include "selectlist.h"
#include "streams.h"
#include "fileman.h"
#include "trackedit.h"
#include "preferences.h"
#include "precachetrack.h"
#include "stdfiletrack.h"

int trackedit_tracks=0;

GtkWidget *trackedit;

int trackedit_streamdrop(char *s)
{
	int success;
	
	tracks_trackinfo *newtrack;
	char sizebuf[20];
	char *trackent[2];
	
	newtrack=streams_getstreambyname((char*)(strchr(s,":"[0])+1));
	if (newtrack!=NULL)
	  {
		  /* claim the track */
		  tracks_claim(newtrack);
		  trackent[0]=(char*)&newtrack->name;
		  sprintf((char*)&sizebuf,"%d",tracks_tracksize(newtrack));
		  trackent[1]=(char*)&sizebuf;
		  gtk_clist_append(GTK_CLIST(trackedit),
				   trackent);
		  gtk_clist_set_row_data(GTK_CLIST(trackedit),
					 trackedit_tracks,
					 newtrack);
		  trackedit_tracks++;
		  success=1;
		  ;
	  }
	else
	    success=0;
	
	return success;	
}
;

/* FIXME: implement filetype detect and use filters to make valid tracks
 * e.g. out of wav files... */
int trackedit_filedrop(char *s)
{
	int success;
	
	tracks_trackinfo *newtrack;
	char sizebuf[20];
	char *trackent[2];

	newtrack=stdfiletrack_create((char*)(strchr(s,":"[0])+1));
	if (newtrack!=NULL)
	  {
		  trackent[0]=(char*)&newtrack->name;
		  sprintf((char*)&sizebuf,"%d",tracks_tracksize(newtrack));
		  trackent[1]=(char*)&sizebuf;
		  gtk_clist_append(GTK_CLIST(trackedit),
				   trackent);
		  gtk_clist_set_row_data(GTK_CLIST(trackedit),
					 trackedit_tracks,
					 newtrack);
		  trackedit_tracks++;
		  success=1;
		  ;
	  }
	else
	    success=0;
	
	return success;	
}
;


int trackedit_trackdrop(char *t,char *p)
{
	int success;
	
	success=0;
	if (trackedit_tracks<MAX_TRACKS)
	  {		  
		  if (!strncmp("stream:",t,7))
		      success=trackedit_streamdrop(t);
		  if (!strncmp("file:",t,5))
		      success=trackedit_filedrop(t);
	  }
	;
	return success;
}
;	

int prereading;         /* TRUE if prereading process running */
int preread_current;

void trackedit_prereadcb(tracks_trackinfo *finished,int success)
{
	char sizebuf[20];
	int nextfound;

	if (success) /* was prereading successful for this track ? */
	  {
		  /* update data in tracklist */
		  gtk_clist_set_row_data(GTK_CLIST(trackedit),
					 preread_current,
					 finished);
		  gtk_clist_set_text(GTK_CLIST(trackedit),
				     preread_current,
				     0,
				     finished->name);
		  sprintf((char*)&sizebuf,"%d",tracks_tracksize(finished));
		  gtk_clist_set_text(GTK_CLIST(trackedit),
				     preread_current,
				     1,
				     (char*)&sizebuf);
	  }
	;

	preread_current++;
	nextfound=0;
	while ((preread_current<trackedit_tracks)&&(!nextfound))
	  {			    
		  if (trackedit_gettrack(preread_current)->precacherequired==atadd)
		    {			    
			    precachetrack_create(trackedit_gettrack(preread_current),
						 trackedit_prereadcb);
			    /* do unclaim the client track in any case,
			     * it will be reclaimed automatically if precaching
			     * is cancelled */
			    tracks_unclaim(trackedit_gettrack(preread_current));
			    nextfound=1;
		    }
		  else
		      preread_current++;
	  }
	;
	prereading=nextfound; /* clear prereading flag if no further tracks */
}
;
		 

void trackedit_doprereading()
{
	int nextfound;
	if (!prereading)
	  {		  
		  preread_current=-1;
		  trackedit_prereadcb(NULL,0);
	  }
	;
}
;
	
void trackedit_drophandler(char *s)
{
	fileman_selectionhandler(s,NULL,trackedit_trackdrop);
	trackedit_doprereading(); /* call the prereading process which steps 
				   * through the whole tracklist and
				   * prereads tracks that have to be preread */
}
;


GtkWidget *trackedit_create()
{
	
	char *fsedit;
	
	selectlist_info *selectlistinfo;
	GtkWidget *scrolling;
	
	scrolling=gtk_scrolled_window_new(NULL,NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolling),
				      GTK_POLICY_AUTOMATIC,
				      GTK_POLICY_AUTOMATIC);	
	
	
	selectlistinfo=selectlist_info_create("stream:",
					      0,
					      trackedit_drophandler,
					      0,0,NULL,
					      NULL, /* no directories,of course ;-) */
					      NULL);
	/* FIXME: currently no edit popup,to be implemented later */
	/*        currently no doubleclick handler */
	trackedit=selectlist_create(selectlistinfo,2);
	
	gtk_clist_set_selection_mode(GTK_CLIST(trackedit),GTK_SELECTION_MULTIPLE);
	gtk_clist_set_column_justification(GTK_CLIST(trackedit),0,GTK_JUSTIFY_LEFT);
	gtk_clist_set_column_width(GTK_CLIST(trackedit),0,256);
	gtk_clist_set_column_justification(GTK_CLIST(trackedit),1,GTK_JUSTIFY_RIGHT);
	gtk_clist_set_column_width(GTK_CLIST(trackedit),1,64);

	gtk_widget_show(trackedit);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolling),
					      trackedit);
	
	return scrolling;
}
;

void trackedit_cleartracklist()
{
	int i;
	for (i=0;i<trackedit_tracks;i++)
	  {
		  tracks_unclaim(trackedit_gettrack(i));
	  }
	;
}
;
	

tracks_trackinfo *trackedit_gettrack(int n)
{
	if (n<trackedit_tracks)
	    return (tracks_trackinfo*)gtk_clist_get_row_data(GTK_CLIST(trackedit),
							     n);
	else
	    return 0;
}
;
	
	
